from .state import _get_translation


class NotAnOutputError(Exception):
    pass


class Event:
    """An item state change event"""

    def __init__(self, item):
        """Initialize the event with the item and the current item state

        Create the event after changing the item state.
        """
        self._item = item
        self._state = item.state

    def __repr__(self):
        return f"{self._item.info()} => {self._state}"

    def __eq__(self, other):
        if other == None:
            return False
        return self._item == other.item and self._state == other.state

    @property
    def item(self):
        """Item this event is related to"""
        return self._item

    @property
    def state(self):
        """State for this event"""
        return self._item


class Item:
    """An item in the Calaos installation"""

    def __init__(self, data, conn, state_translator):
        self._id = data["id"]
        self._gui_type = data["gui_type"]
        self._io_type = data["io_type"]
        self._name = data["name"]
        self._state = state_translator.parse(data["state"])
        self._type = data["type"]
        self._var_type = data["var_type"]
        self._visible = data["visible"] == "true"
        self._conn = conn
        self._state_translator = state_translator

    def info(self):
        return f"[{self._id}] {self._name} ({self._type}/{self._io_type})"

    def __repr__(self):
        return f"{self.info()} = {self._state}"

    @property
    def id(self):
        return self._id

    @property
    def gui_type(self):
        return self._gui_type

    @property
    def io_type(self):
        return self._io_type

    @property
    def name(self):
        return self._name

    @property
    def state(self):
        return self._state

    @property
    def type(self):
        return self._type

    @property
    def var_type(self):
        return self._var_type

    @property
    def visible(self):
        return self._visible

    def _update_state(self, state):
        newState = self._state_translator.parse(state)
        if newState == self._state:
            return None
        self._state = newState
        return Event(self)

    @state.setter
    def state(self, state):
        if self._io_type == "output" or self._io_type == "inout":
            newState = self._state_translator.parse(state)
            self._conn.send({
                "action": "set_state",
                "type": self._type,
                "id": self._id,
                "value": self._state_translator.raw(newState)
            })
            self._state = newState
        else:
            raise NotAnOutputError


def _newItem(data, conn):
    return Item(data, conn, _get_translation(data["type"]))



from enum import Enum


class generic:
    def parse(state):
        return state

    def raw(state):
        return str(state)


class binary(generic):
    def parse(state):
        return state == "true" or state == True

    def raw(state):
        return "true" if state else "false"


class NbClicks(Enum):
    NONE = 0
    SINGLE = 1
    DOUBLE = 2
    TRIPLE = 3


class up_to_triple(generic):
    def parse(state):
        return NbClicks(int(state))

    def raw(state):
        return str(state.value)


class ClickType(Enum):
    NONE = 0
    SHORT = 1
    LONG = 2


class short_or_long(generic):
    def parse(state):
        return ClickType(int(state))

    def raw(state):
        return str(state.value)


class Running(Enum):
    NOT_RUNNING = 0
    RUNNING = 1


class running(generic):
    def parse(state):
        return Running.RUNNING if state == "true" else Running.NOT_RUNNING

    def raw(state):
        return state == Running.RUNNING


class percentage(generic):
    def parse(state):
        if state == True:
            return 100
        if state == False:
            return 0
        value = int(state)
        if value > 100:
            value = 100
        elif value < 0:
            value = 0
        return value


_translation_map = {
    "InPlageHoraire": binary,
    "scenario": running,
    "WIDigitalBP": binary,
    "WIDigitalLong": short_or_long,
    "WIDigitalTriple": up_to_triple,
    "WODali": percentage,
    "WODigital": binary
}


def _get_translation(name):
    try:
        return _translation_map[name]
    except KeyError:
        return generic
